@@ -25,16 +25,14 @@ struct Avatar
     mtx::common::ImageInfo image_info;
     //! The URL to the image.
     std::string url;
+
+    //! Deserialization method needed by @p nlohmann::json.
+    friend void from_json(const nlohmann::json &obj, Avatar &avatar);
+
+    //! Serialization method needed by @p nlohmann::json.
+    friend void to_json(nlohmann::json &obj, const Avatar &avatar);
 };
 
-//! Deserialization method needed by @p nlohmann::json.
-void
-from_json(const nlohmann::json &obj, Avatar &avatar);
-
-//! Serialization method needed by @p nlohmann::json.
-void
-to_json(nlohmann::json &obj, const Avatar &avatar);
-
 } // namespace state
 } // namespace events
 } // namespace mtx
