@@ -18,7 +18,7 @@ namespace mtx {
 //! Common structs used among many different event content types.
 namespace common {
 
-constexpr auto FORMAT_MSG_TYPE = "org.matrix.custom.html";
+inline constexpr auto FORMAT_MSG_TYPE = "org.matrix.custom.html";
 
 //! Metadata about an image thumbnail.
 struct ThumbnailInfo
@@ -152,6 +152,33 @@ struct LocationInfo
     friend void to_json(nlohmann::json &obj, const ThumbnailInfo &info);
 };
 
+/// @brief Mentions metadata
+///
+/// @sa https://spec.matrix.org/v1.9/client-server-api/#user-and-room-mentions
+struct Mentions
+{
+    std::vector<std::string> user_ids;
+    bool room = false;
+
+    //! Deserialization method needed by @p nlohmann::json.
+    friend void from_json(const nlohmann::json &obj, Mentions &info);
+
+    //! Serialization method needed by @p nlohmann::json.
+    friend void to_json(nlohmann::json &obj, const Mentions &info);
+};
+
+/// @brief Parses mentions from a content object
+///
+/// @param obj The content object of an event.
+std::optional<Mentions>
+parse_mentions(const nlohmann::json &obj);
+
+/// @brief Serializes mentions to a content object
+///
+/// @param obj The content object of an event.
+void
+add_mentions(nlohmann::json &content, const std::optional<Mentions> &m);
+
 //! Definition of rel_type for relations.
 enum class RelationType
 {
@@ -163,6 +190,8 @@ enum class RelationType
     Replace,
     //! im.nheko.relations.v1.in_reply_to rel_type
     InReplyTo,
+    //! m.thread
+    Thread,
     //! not one of the supported types
     Unsupported
 };
@@ -183,6 +212,9 @@ struct Relation
     //! key is the reaction itself
     std::optional<std::string> key = std::nullopt;
 
+    //! proprietary field to track if this is a fallback for something else
+    bool is_fallback = false;
+
     friend void from_json(const nlohmann::json &obj, Relation &relation);
     friend void to_json(nlohmann::json &obj, const Relation &relation);
 };
@@ -196,10 +228,11 @@ struct Relations
     //! im.nheko.relactions.v1.relations
     bool synthesized = false;
 
-    std::optional<std::string> reply_to() const;
-    std::optional<std::string> replaces() const;
-    std::optional<std::string> references() const;
-    std::optional<Relation> annotates() const;
+    [[nodiscard]] std::optional<std::string> reply_to(bool include_fallback = true) const;
+    [[nodiscard]] std::optional<std::string> replaces(bool include_fallback = true) const;
+    [[nodiscard]] std::optional<std::string> references(bool include_fallback = true) const;
+    [[nodiscard]] std::optional<std::string> thread(bool include_fallback = true) const;
+    [[nodiscard]] std::optional<Relation> annotates(bool include_fallback = true) const;
 };
 
 /// @brief Parses relations from a content object
