@@ -6,7 +6,6 @@
 
 #include <exception>
 #include <memory>
-#include <new>
 
 #if __has_include(<nlohmann/json_fwd.hpp>)
 #include <nlohmann/json_fwd.hpp>
@@ -17,6 +16,7 @@
 #include <mtx/identifiers.hpp>
 #include <mtx/requests.hpp>
 
+#include <olm/error.h>
 #include <olm/olm.h>
 #include <olm/sas.h>
 
@@ -27,93 +27,97 @@
 namespace mtx {
 //! Cryptography related types
 namespace crypto {
+using OlmErrorCode = ::OlmErrorCode;
 
 //! Data representation used to interact with libolm.
 using BinaryBuf = std::vector<uint8_t>;
 
-enum class OlmErrorCode
-{
-    UNKNOWN_ERROR = -1,
-    SUCCESS,
-    NOT_ENOUGH_RANDOM,
-    OUTPUT_BUFFER_TOO_SMALL,
-    BAD_MESSAGE_VERSION,
-    BAD_MESSAGE_FORMAT,
-    BAD_MESSAGE_MAC,
-    BAD_MESSAGE_KEY_ID,
-    INVALID_BASE64,
-    BAD_ACCOUNT_KEY,
-    UNKNOWN_PICKLE_VERSION,
-    CORRUPTED_PICKLE,
-    BAD_SESSION_KEY,
-    UNKNOWN_MESSAGE_INDEX,
-    BAD_LEGACY_ACCOUNT_PICKLE,
-    BAD_SIGNATURE,
-    OLM_INPUT_BUFFER_TOO_SMALL,
-    OLM_SAS_THEIR_KEY_NOT_SET
-};
-
 //! Errors returned by the olm library
 class olm_exception : public std::exception
 {
 public:
     olm_exception(std::string func, OlmSession *s)
-      : olm_exception(std::move(func), std::string(olm_session_last_error(s)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_session_last_error(s)),
+                      olm_session_last_error_code(s))
+    {
+    }
 
     olm_exception(std::string func, OlmAccount *acc)
       : olm_exception(std::move(func),
-                      std::string(acc ? olm_account_last_error(acc) : "account == nullptr"))
-    {}
+                      std::string(acc ? olm_account_last_error(acc) : "account == nullptr"),
+                      acc ? olm_account_last_error_code(acc) : UNKNOWN_ERROR)
+    {
+    }
 
     olm_exception(std::string func, OlmUtility *util)
-      : olm_exception(std::move(func), std::string(olm_utility_last_error(util)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_utility_last_error(util)),
+                      olm_utility_last_error_code(util))
+    {
+    }
 
     olm_exception(std::string func, OlmPkDecryption *s)
-      : olm_exception(std::move(func), std::string(olm_pk_decryption_last_error(s)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_pk_decryption_last_error(s)),
+                      olm_pk_decryption_last_error_code(s))
+    {
+    }
 
     olm_exception(std::string func, OlmPkEncryption *s)
-      : olm_exception(std::move(func), std::string(olm_pk_encryption_last_error(s)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_pk_encryption_last_error(s)),
+                      olm_pk_encryption_last_error_code(s))
+    {
+    }
 
     olm_exception(std::string func, OlmPkSigning *s)
-      : olm_exception(std::move(func), std::string(olm_pk_signing_last_error(s)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_pk_signing_last_error(s)),
+                      olm_pk_signing_last_error_code(s))
+    {
+    }
 
     olm_exception(std::string func, OlmOutboundGroupSession *s)
       : olm_exception(
           std::move(func),
-          std::string(s ? olm_outbound_group_session_last_error(s) : "session == nullptr"))
-    {}
+          std::string(s ? olm_outbound_group_session_last_error(s) : "session == nullptr"),
+          s ? olm_outbound_group_session_last_error_code(s) : UNKNOWN_ERROR)
+    {
+    }
 
     olm_exception(std::string func, OlmInboundGroupSession *s)
       : olm_exception(
           std::move(func),
-          std::string(s ? olm_inbound_group_session_last_error(s) : "session == nullptr"))
-    {}
+          std::string(s ? olm_inbound_group_session_last_error(s) : "session == nullptr"),
+          s ? olm_inbound_group_session_last_error_code(s) : UNKNOWN_ERROR)
+    {
+    }
 
     olm_exception(std::string func, OlmSAS *s)
-      : olm_exception(std::move(func), std::string(olm_sas_last_error(s)))
-    {}
+      : olm_exception(std::move(func),
+                      std::string(olm_sas_last_error(s)),
+                      olm_sas_last_error_code(s))
+    {
+    }
 
     //! Returns a description of the olm error.
-    const char *what() const noexcept override { return msg_.c_str(); }
+    [[nodiscard]] const char *what() const noexcept override { return msg_.c_str(); }
 
     //! Returns an error code reconstructed from the error string returned by olm
-    OlmErrorCode error_code() const noexcept { return ec_; }
+    [[nodiscard]] OlmErrorCode error_code() const noexcept { return ec_; }
+
+    static constexpr OlmErrorCode UNKNOWN_ERROR = OLM_SUCCESS;
 
 private:
-    olm_exception(std::string &&func, std::string error_string)
+    olm_exception(std::string &&func, std::string &&error_string, OlmErrorCode code)
       : msg_(func + ": " + error_string)
-      , ec_(ec_from_string(error_string))
-    {}
-
-    OlmErrorCode ec_from_string(std::string_view);
+      , ec_(code)
+    {
+    }
 
     std::string msg_;
-    OlmErrorCode ec_ = OlmErrorCode::UNKNOWN_ERROR;
+    OlmErrorCode ec_ = (OlmErrorCode)OLM_SUCCESS;
 };
 
 //! Serialize olm objects into strings encrypted using key to persist them on non volatile storage.
@@ -176,10 +180,26 @@ struct SAS
     /// Map these numpers to one of the 64 emoji from the specification and let the user compare
     /// them.
     std::vector<int> generate_bytes_emoji(const std::string &info);
+
     //! Calculate MACs after verification to verify keys.
-    std::string calculate_mac(const std::string &input_data, const std::string &info);
+    //! mac_version is either hkdf-hmac-sha256 or hkdf-hmac-sha256.v2
+    //! keys are the keys to verify.
+    //! Transaction id is the transaction id of the current verification exchange.
+    //! *Device are the device ids of the respective senders.
+    mtx::events::msg::KeyVerificationMac calculate_mac(
+      std::string_view mac_version,
+      const mtx::identifiers::User &sender,
+      const std::string &senderDevice,
+      const mtx::identifiers::User &receiver,
+      const std::string &receiverDevice,
+      const std::string &transactionId,
+      const std::map<std::string, std::string> &keys);
 
 private:
+    std::string calculate_mac_v(const std::string &input_data,
+                                const std::string &info,
+                                std::string_view mac_version);
+
     SASPtr sas;
 };
 
@@ -216,7 +236,8 @@ public:
     OlmClient(std::string user_id, std::string device_id)
       : user_id_(std::move(user_id))
       , device_id_(std::move(device_id))
-    {}
+    {
+    }
 
     //! Base64 encoded string
     using Base64String = std::string;
@@ -276,22 +297,30 @@ public:
     Base64String sign_identity_keys();
 
     //! Generate a number of one time keys.
-    std::size_t generate_one_time_keys(std::size_t nkeys);
+    std::size_t generate_one_time_keys(std::size_t nkeys, bool generate_fallback = false);
     //! Retrieve the json representation of the one time keys for the given account.
     OneTimeKeys one_time_keys();
+    //! Retrieve the json representation of the unpublished fallback one time keys for the given
+    //! account.
+    OneTimeKeys unpublished_fallback_keys();
     //! Sign the given one time keys and encode it to base64.
-    Base64String sign_one_time_key(const Base64String &encoded_key);
+    Base64String sign_one_time_key(const Base64String &encoded_key, bool fallback = false);
     //! Sign one_time_keys and generate the appropriate structure for the /keys/upload request.
-    SignedOneTimeKeys sign_one_time_keys(const OneTimeKeys &keys);
+    SignedOneTimeKeys sign_one_time_keys(const OneTimeKeys &keys, bool fallback = false);
     //! Generate the json structure for the signed one time key.
     requests::SignedOneTimeKey signed_one_time_key(const std::string &key,
-                                                   const std::string &signature);
+                                                   const std::string &signature,
+                                                   bool fallback = false);
 
     //! Marks the current set of one time keys as being published.
     void mark_keys_as_published() { olm_account_mark_keys_as_published(account_.get()); }
+    //! Forgets an old fallback key. Call this when you are sure the old key is no longer in use,
+    //! i.e. 5 minutes after publishing a new one.
+    void forget_old_fallback_key() { olm_account_forget_old_fallback_key(account_.get()); }
 
     //! Prepare request for the /keys/upload endpoint by signing identity & one time keys.
-    mtx::requests::UploadKeys create_upload_keys_request(const OneTimeKeys &keys);
+    mtx::requests::UploadKeys create_upload_keys_request(const OneTimeKeys &keys,
+                                                         const OneTimeKeys &fallback_keys);
     //! Prepare an empty /keys/upload request.
     mtx::requests::UploadKeys create_upload_keys_request();
 
