@@ -74,6 +74,8 @@ getEventType(const std::string &type)
         return EventType::RoomPinnedEvents;
     else if (type == "m.room.tombstone")
         return EventType::RoomTombstone;
+    else if (type == "m.room.server_acl")
+        return EventType::RoomServerAcl;
     else if (type == "m.sticker")
         return EventType::Sticker;
 
@@ -121,14 +123,20 @@ getEventType(const std::string &type)
         return EventType::Receipt;
     else if (type == "m.fully_read")
         return EventType::FullyRead;
+    else if (type == "m.ignored_user_list")
+        return EventType::IgnoredUsers;
     else if (type == "im.nheko.hidden_events")
         return EventType::NhekoHiddenEvents;
+    else if (type == "im.nheko.event_expiry")
+        return EventType::NhekoEventExpiry;
     else if (type == "im.ponies.room_emotes")
         return EventType::ImagePackInRoom;
     else if (type == "im.ponies.user_emotes")
         return EventType::ImagePackInAccountData;
     else if (type == "im.ponies.emote_rooms")
         return EventType::ImagePackRooms;
+    else if (type == "")
+        throw std::invalid_argument("Empty event type");
     else
         return EventType::Unsupported;
 }
@@ -201,6 +209,8 @@ to_string(EventType type)
         return "m.room.pinned_events";
     case EventType::RoomTombstone:
         return "m.room.tombstone";
+    case EventType::RoomServerAcl:
+        return "m.room.server_acl";
     case EventType::Sticker:
         return "m.sticker";
 
@@ -248,8 +258,12 @@ to_string(EventType type)
         return "m.receipt";
     case EventType::FullyRead:
         return "m.fully_read";
+    case EventType::IgnoredUsers:
+        return "m.ignored_user_list";
     case EventType::NhekoHiddenEvents:
         return "im.nheko.hidden_events";
+    case EventType::NhekoEventExpiry:
+        return "im.nheko.event_expiry";
     case EventType::ImagePackInRoom:
         return "im.ponies.room_emotes";
     case EventType::ImagePackInAccountData:
@@ -289,8 +303,10 @@ getMessageType(const std::string &type)
         return MessageType::Notice;
     else if (type == "m.text")
         return MessageType::Text;
-    else if (type == "nic.custom.confetti")
-        return MessageType::Confetti;
+    else if (type == "nic.custom.confetti" || type == "nic.custom.fireworks" ||
+             type == "io.element.effect.rainfall" || type == "io.element.effect.hearts" ||
+             type == "io.element.effect.snowfall" || type == "io.element.effects.space_invaders")
+        return MessageType::ElementEffect;
     else if (type == "m.video")
         return MessageType::Video;
     else if (type == "m.key.verification.request")
@@ -303,15 +319,15 @@ MessageType
 getMessageType(const json &obj)
 {
     if (obj.is_null())
-        return MessageType::Unknown;
+        return MessageType::Redacted;
 
     if (obj.find("msgtype") == obj.end())
-        return MessageType::Unknown;
+        return MessageType::Invalid;
 
     try {
         return getMessageType(obj.at("msgtype").get<std::string>());
     } catch (...) {
-        return MessageType::Unknown;
+        return MessageType::Invalid;
     }
 }
 }
