@@ -30,6 +30,7 @@ from_json(const json &obj, File &content)
         content.file = obj.at("file").get<crypto::EncryptedFile>();
 
     content.relations = common::parse_relations(obj);
+    content.mentions  = common::parse_mentions(obj);
 }
 
 void
@@ -47,6 +48,7 @@ to_json(json &obj, const File &content)
     else
         obj["url"] = content.url;
 
+    common::add_mentions(obj, content.mentions);
     common::apply_relations(obj, content.relations);
 }
 
