@@ -12,14 +12,14 @@ namespace state {
 void
 from_json(const json &obj, Name &event)
 {
-        if (!obj.at("name").is_null())
-                event.name = obj.at("name").get<std::string>();
+    if (obj.find("name") != obj.end() && !obj.at("name").is_null())
+        event.name = obj.at("name").get<std::string>();
 }
 
 void
 to_json(json &obj, const Name &event)
 {
-        obj["name"] = event.name;
+    obj["name"] = event.name;
 }
 
 } // namespace state
