@@ -8,15 +8,41 @@
 #include "mtx/events/collections.hpp"
 #include "mtx/log.hpp"
 
+using non_compound_json_value = std::variant<std::string, std::int64_t, bool, std::nullptr_t>;
+using push_json_value         = std::
+  variant<std::string, std::int64_t, bool, std::nullptr_t, std::vector<non_compound_json_value>>;
+
 namespace {
 struct RelatedEvents
 {
-    std::vector<std::unordered_map<std::string, std::string>>
-      fallbacks;                                                      //!< fallback related events
-    std::vector<std::unordered_map<std::string, std::string>> events; //!< related events
+    std::vector<std::unordered_map<std::string, push_json_value>>
+      fallbacks; //!< fallback related events
+    std::vector<std::unordered_map<std::string, push_json_value>> events; //!< related events
 };
 }
 
+static std::optional<non_compound_json_value>
+to_non_compound_json_value(const nlohmann::json &val)
+{
+    switch (val.type()) {
+    case nlohmann::json::value_t::string: {
+        return val.get<std::string>();
+    }
+    case nlohmann::json::value_t::null: {
+        return nullptr;
+    }
+    case nlohmann::json::value_t::boolean: {
+        return val.get<bool>();
+    }
+    case nlohmann::json::value_t::number_integer:
+    case nlohmann::json::value_t::number_unsigned: {
+        return val.get<std::int64_t>();
+    }
+    default:
+        return std::nullopt;
+    }
+}
+
 namespace mtx {
 namespace pushrules {
 
@@ -32,6 +58,10 @@ to_json(nlohmann::json &obj, const PushCondition &condition)
         obj["is"] = condition.is;
     if (condition.rel_type != mtx::common::RelationType::Unsupported)
         obj["rel_type"] = condition.rel_type;
+
+    if (condition.value) {
+        std::visit([&obj](const auto &e) { obj["value"] = e; }, *condition.value);
+    }
 }
 
 void
@@ -43,6 +73,10 @@ from_json(const nlohmann::json &obj, PushCondition &condition)
     condition.is               = obj.value("is", "");
     condition.rel_type         = obj.value("rel_type", mtx::common::RelationType::Unsupported);
     condition.include_fallback = obj.value("include_fallback", false);
+
+    if (obj.contains("value"))
+        if (auto val = to_non_compound_json_value(obj.at("value")))
+            condition.value = *val;
 }
 
 namespace actions {
@@ -196,15 +230,17 @@ struct PushRuleEvaluator::OptimizedRules
             std::unique_ptr<re2::RE2> pattern; //!< the pattern
             std::string field;                 //!< the field to match with pattern
 
-            bool matches(const std::unordered_map<std::string, std::string> &ev) const
+            [[nodiscard]] bool matches(
+              const std::unordered_map<std::string, push_json_value> &ev) const
             {
                 if (auto it = ev.find(field); it != ev.end()) {
-                    if (pattern) {
+                    if (pattern && std::holds_alternative<std::string>(it->second)) {
                         if (field == "content.body") {
-                            if (!re2::RE2::PartialMatch(it->second, *pattern))
+                            if (!re2::RE2::PartialMatch(std::get<std::string>(it->second),
+                                                        *pattern))
                                 return false;
                         } else {
-                            if (!re2::RE2::FullMatch(it->second, *pattern))
+                            if (!re2::RE2::FullMatch(std::get<std::string>(it->second), *pattern))
                                 return false;
                         }
                     }
@@ -212,12 +248,54 @@ struct PushRuleEvaluator::OptimizedRules
                     return false;
                 }
 
+                // We have some internal rules, which just match on a field being present.
                 return true;
             }
         };
         // TODO(Nico): Sort by field for faster matching?
         std::vector<PatternCondition> patterns; //!< conditions that match on a field
 
+        //! a event_property_is condition to match
+        struct IsCondition
+        {
+            non_compound_json_value value; //!< the pattern
+            std::string field;             //!< the field to match with pattern
+
+            [[nodiscard]] bool matches(
+              const std::unordered_map<std::string, push_json_value> &ev) const
+            {
+                if (auto it = ev.find(field); it != ev.end()) {
+                    return std::visit(
+                      [it](const auto &e) { return it->second == push_json_value(e); }, value);
+                }
+
+                return false;
+            }
+        };
+        std::vector<IsCondition> is; //!< conditions that match on a field being an exact value
+                                     //!
+        //! a event_property_contains condition to match
+        struct ContainsCondition
+        {
+            non_compound_json_value value; //!< the pattern
+            std::string field;             //!< the field to match with pattern
+
+            [[nodiscard]] bool matches(
+              const std::unordered_map<std::string, push_json_value> &ev) const
+            {
+                if (auto it = ev.find(field); it != ev.end()) {
+                    if (auto arr = std::get_if<std::vector<non_compound_json_value>>(&it->second)) {
+                        return std::find(arr->begin(), arr->end(), value) != arr->end();
+                    }
+                }
+
+                return false;
+            }
+        };
+        std::vector<ContainsCondition> contains; //!< conditions that match on arrays
+
+        bool no_mentions_field = false; //!< rule only matches if no content.m\\.mentions is present
+
         //! a pattern condition to match on a related event
         struct RelatedEventCondition
         {
@@ -255,10 +333,14 @@ struct PushRuleEvaluator::OptimizedRules
         std::vector<actions::Action> actions; //< the actions to apply on match
 
         [[nodiscard]] bool matches(
-          const std::unordered_map<std::string, std::string> &ev,
+          const std::unordered_map<std::string, push_json_value> &ev,
           const PushRuleEvaluator::RoomContext &ctx,
           const std::map<mtx::common::RelationType, RelatedEvents> &relatedEventsFlat) const
         {
+            if (no_mentions_field && ev.contains("content.m\\.mentions")) {
+                return false;
+            }
+
             for (const auto &cond : membercounts) {
                 if (![&cond, &ctx] {
                         switch (cond.op) {
@@ -284,7 +366,8 @@ struct PushRuleEvaluator::OptimizedRules
                 if (sender_ == ev.end())
                     return false;
 
-                auto sender_level = ctx.power_levels.user_level(sender_->second);
+                auto sender_level =
+                  ctx.power_levels.user_level(std::get<std::string>(sender_->second));
 
                 for (const auto &n : notification_levels) {
                     if (sender_level < ctx.power_levels.notification_level(n))
@@ -296,6 +379,14 @@ struct PushRuleEvaluator::OptimizedRules
                 if (!cond.matches(ev))
                     return false;
             }
+            for (const auto &cond : is) {
+                if (!cond.matches(ev))
+                    return false;
+            }
+            for (const auto &cond : contains) {
+                if (!cond.matches(ev))
+                    return false;
+            }
 
             for (const auto &cond : related_event_patterns) {
                 bool matched = false;
@@ -327,12 +418,13 @@ struct PushRuleEvaluator::OptimizedRules
                 if (ctx.user_display_name.empty())
                     return false;
 
-                if (auto it = ev.find("content.body"); it != ev.end()) {
+                if (auto it = ev.find("content.body");
+                    it != ev.end() && std::holds_alternative<std::string>(it->second)) {
                     re2::RE2::Options opts;
                     opts.set_case_sensitive(false);
 
                     if (!re2::RE2::PartialMatch(
-                          it->second,
+                          std::get<std::string>(it->second),
                           re2::RE2("(\\W|^)" + re2::RE2::QuoteMeta(ctx.user_display_name) +
                                      "(\\W|$)",
                                    opts)))
@@ -378,67 +470,97 @@ PushRuleEvaluator::~PushRuleEvaluator() = default;
 PushRuleEvaluator::PushRuleEvaluator(const Ruleset &rules_)
   : rules(std::make_unique<OptimizedRules>())
 {
-    auto add_conditions_to_rule = [](OptimizedRules::OptimizedRule &rule,
-                                     const std::vector<PushCondition> &conditions) {
-        for (const auto &cond : conditions) {
-            if (cond.kind == "event_match") {
-                OptimizedRules::OptimizedRule::PatternCondition c;
-                c.field   = cond.key;
-                c.pattern = construct_re_from_pattern(cond.pattern, cond.key);
-                if (c.pattern)
-                    rule.patterns.push_back(std::move(c));
-            } else if (cond.kind == "im.nheko.msc3664.related_event_match") {
-                OptimizedRules::OptimizedRule::RelatedEventCondition c;
-
-                if (cond.rel_type != mtx::common::RelationType::Unsupported) {
-                    c.rel_type          = cond.rel_type;
-                    c.include_fallbacks = cond.include_fallback;
-
-                    if (!cond.key.empty() && !cond.pattern.empty()) {
-                        c.ev_match.field   = cond.key;
-                        c.ev_match.pattern = construct_re_from_pattern(cond.pattern, cond.key);
-                    }
-                    rule.related_event_patterns.push_back(std::move(c));
-                } else {
-                    mtx::utils::log::log()->info(
-                      "Skipping rel_event_match rule with unknown rel_type.");
-                    return false;
-                }
-            } else if (cond.kind == "contains_display_name") {
-                rule.check_displayname = true;
-            } else if (cond.kind == "room_member_count") {
-                OptimizedRules::OptimizedRule::MemberCountCondition c;
-                std::string_view is = cond.is;
-                if (is.starts_with("==")) {
-                    c.op = c.Comp::Eq;
-                    is   = is.substr(2);
-                } else if (is.starts_with(">=")) {
-                    c.op = c.Comp::Ge;
-                    is   = is.substr(2);
-                } else if (is.starts_with("<=")) {
-                    c.op = c.Comp::Le;
-                    is   = is.substr(2);
-                } else if (is.starts_with('<')) {
-                    c.op = c.Comp::Lt;
-                    is   = is.substr(1);
-                } else if (is.starts_with('>')) {
-                    c.op = c.Comp::Gt;
-                    is   = is.substr(1);
-                }
-
-                std::from_chars(is.data(), is.data() + is.size(), c.count);
-                rule.membercounts.push_back(c);
-            } else if (cond.kind == "sender_notification_permission") {
-                rule.notification_levels.push_back(cond.key);
-            } else {
-                mtx::utils::log::log()->info("Skipping rule with unknown condition type: {}",
-                                             cond.kind);
-                return false;
-            }
+    // In theory we should check the server version, but we can't access that here and this should
+    // work on all servers for now.
+    bool server_supports_mentions = false;
+    for (const auto &r : rules_.override_) {
+        if (r.rule_id == ".m.rule.is_user_mention") {
+            server_supports_mentions = true;
+            break;
         }
+    }
 
-        return true;
-    };
+    auto add_conditions_to_rule =
+      [server_supports_mentions](OptimizedRules::OptimizedRule &rule,
+                                 const std::vector<PushCondition> &conditions,
+                                 std::string_view rule_id) {
+          // These rules should only be enabled, if there is no m.mentions property, but the spec
+          // doesn't actually add such a condition, so let's do it ourselves...
+          if (server_supports_mentions &&
+              (rule_id == ".m.rule.contains_display_name" || rule_id == ".m.rule.roomnotif" ||
+               rule_id == ".m.rule.contains_user_name")) {
+              rule.no_mentions_field = true;
+          }
+
+          for (const auto &cond : conditions) {
+              if (cond.kind == "event_match") {
+                  OptimizedRules::OptimizedRule::PatternCondition c;
+                  c.field   = cond.key;
+                  c.pattern = construct_re_from_pattern(cond.pattern, cond.key);
+                  if (c.pattern)
+                      rule.patterns.push_back(std::move(c));
+              } else if (cond.kind == "event_property_is" && cond.value) {
+                  OptimizedRules::OptimizedRule::IsCondition c;
+                  c.field = cond.key;
+                  c.value = cond.value.value();
+                  rule.is.push_back(std::move(c));
+              } else if (cond.kind == "event_property_contains" && cond.value) {
+                  OptimizedRules::OptimizedRule::ContainsCondition c;
+                  c.field = cond.key;
+                  c.value = cond.value.value();
+                  rule.contains.push_back(std::move(c));
+              } else if (cond.kind == "im.nheko.msc3664.related_event_match") {
+                  OptimizedRules::OptimizedRule::RelatedEventCondition c;
+
+                  if (cond.rel_type != mtx::common::RelationType::Unsupported) {
+                      c.rel_type          = cond.rel_type;
+                      c.include_fallbacks = cond.include_fallback;
+
+                      if (!cond.key.empty() && !cond.pattern.empty()) {
+                          c.ev_match.field   = cond.key;
+                          c.ev_match.pattern = construct_re_from_pattern(cond.pattern, cond.key);
+                      }
+                      rule.related_event_patterns.push_back(std::move(c));
+                  } else {
+                      mtx::utils::log::log()->info(
+                        "Skipping rel_event_match rule with unknown rel_type.");
+                      return false;
+                  }
+              } else if (cond.kind == "contains_display_name") {
+                  rule.check_displayname = true;
+              } else if (cond.kind == "room_member_count") {
+                  OptimizedRules::OptimizedRule::MemberCountCondition c;
+                  std::string_view is = cond.is;
+                  if (is.starts_with("==")) {
+                      c.op = c.Comp::Eq;
+                      is   = is.substr(2);
+                  } else if (is.starts_with(">=")) {
+                      c.op = c.Comp::Ge;
+                      is   = is.substr(2);
+                  } else if (is.starts_with("<=")) {
+                      c.op = c.Comp::Le;
+                      is   = is.substr(2);
+                  } else if (is.starts_with('<')) {
+                      c.op = c.Comp::Lt;
+                      is   = is.substr(1);
+                  } else if (is.starts_with('>')) {
+                      c.op = c.Comp::Gt;
+                      is   = is.substr(1);
+                  }
+
+                  std::from_chars(is.data(), is.data() + is.size(), c.count);
+                  rule.membercounts.push_back(c);
+              } else if (cond.kind == "sender_notification_permission") {
+                  rule.notification_levels.push_back(cond.key);
+              } else {
+                  mtx::utils::log::log()->info("Skipping rule with unknown condition type: {}",
+                                               cond.kind);
+                  return false;
+              }
+          }
+
+          return true;
+      };
 
     for (const auto &rule_ : rules_.override_) {
         if (!rule_.enabled)
@@ -447,7 +569,7 @@ PushRuleEvaluator::PushRuleEvaluator(const Ruleset &rules_)
         OptimizedRules::OptimizedRule rule;
         rule.actions = rule_.actions;
 
-        if (!add_conditions_to_rule(rule, rule_.conditions))
+        if (!add_conditions_to_rule(rule, rule_.conditions, rule_.rule_id))
             continue;
 
         rules->override_.push_back(std::move(rule));
@@ -460,7 +582,7 @@ PushRuleEvaluator::PushRuleEvaluator(const Ruleset &rules_)
         OptimizedRules::OptimizedRule rule;
         rule.actions = rule_.actions;
 
-        if (!add_conditions_to_rule(rule, rule_.conditions))
+        if (!add_conditions_to_rule(rule, rule_.conditions, rule_.rule_id))
             continue;
 
         rules->underride.push_back(std::move(rule));
@@ -507,7 +629,7 @@ PushRuleEvaluator::PushRuleEvaluator(const Ruleset &rules_)
           PushCondition{.kind = "event_match", .key = "content.body", .pattern = rule_.pattern},
         };
 
-        if (!add_conditions_to_rule(rule, conditions))
+        if (!add_conditions_to_rule(rule, conditions, rule_.rule_id))
             continue;
 
         rules->content.push_back(std::move(rule));
@@ -516,38 +638,90 @@ PushRuleEvaluator::PushRuleEvaluator(const Ruleset &rules_)
 
 static void
 flatten_impl(const nlohmann::json &value,
-             std::unordered_map<std::string, std::string> &result,
+             std::unordered_map<std::string, push_json_value> &result,
              const std::string &current_path,
              int current_depth)
 {
     if (current_depth > 100)
         return;
 
+    auto escape_key = [](std::string input) {
+        for (size_t i = 0; i < input.size(); i++) {
+            if (input[i] == '.') {
+                input.insert(i, 1, '\\');
+                i++;
+            }
+        }
+        return input;
+    };
+
     switch (value.type()) {
     case nlohmann::json::value_t::object: {
+        // Workaround to be able to check, if the m.mentions object is present
+        if (current_path == "content.m\\.mentions") {
+            result[current_path] = nullptr;
+        }
+
         // iterate object and use keys as reference string
         std::string prefix;
         if (!current_path.empty())
             prefix = current_path + ".";
         for (const auto &element : value.items()) {
-            flatten_impl(element.value(), result, prefix + element.key(), current_depth + 1);
+            flatten_impl(
+              element.value(), result, prefix + escape_key(element.key()), current_depth + 1);
         }
         break;
     }
 
-    case nlohmann::json::value_t::string: {
-        // add primitive value with its reference string
-        result[current_path] = value.get<std::string>();
+    case nlohmann::json::value_t::array: {
+        std::vector<non_compound_json_value> arr;
+        for (const auto &val : value) {
+            switch (val.type()) {
+            case nlohmann::json::value_t::string: {
+                arr.emplace_back(val.get<std::string>());
+                break;
+            }
+            case nlohmann::json::value_t::null: {
+                arr.emplace_back(nullptr);
+                break;
+            }
+            case nlohmann::json::value_t::boolean: {
+                arr.emplace_back(val.get<bool>());
+                break;
+            }
+            case nlohmann::json::value_t::number_integer:
+            case nlohmann::json::value_t::number_unsigned: {
+                arr.emplace_back(val.get<std::int64_t>());
+                break;
+            }
+            default:
+                break;
+            }
+        }
+
+        result[current_path] = arr;
         break;
     }
 
-        // currently we only match strings
-    case nlohmann::json::value_t::array:
-    case nlohmann::json::value_t::null:
-    case nlohmann::json::value_t::boolean:
+    case nlohmann::json::value_t::string: {
+        result[current_path] = value.get<std::string>();
+        break;
+    }
+    case nlohmann::json::value_t::null: {
+        result[current_path] = nullptr;
+        break;
+    }
+    case nlohmann::json::value_t::boolean: {
+        result[current_path] = value.get<bool>();
+        break;
+    }
     case nlohmann::json::value_t::number_integer:
-    case nlohmann::json::value_t::number_unsigned:
-    case nlohmann::json::value_t::number_float:
+    case nlohmann::json::value_t::number_unsigned: {
+        result[current_path] = value.get<std::int64_t>();
+        break;
+    }
+
+    case nlohmann::json::value_t::number_float: // matrix events can't have floats
     case nlohmann::json::value_t::binary:
     case nlohmann::json::value_t::discarded:
     default:
@@ -555,19 +729,19 @@ flatten_impl(const nlohmann::json &value,
     }
 }
 
-static std::unordered_map<std::string, std::string>
+static std::unordered_map<std::string, push_json_value>
 flatten_event(const nlohmann::json &j)
 {
-    std::unordered_map<std::string, std::string> flat;
+    std::unordered_map<std::string, push_json_value> flat;
     flatten_impl(j, flat, "", 0);
     return flat;
 }
 
 std::vector<actions::Action>
 PushRuleEvaluator::evaluate(
-  const mtx::events::collections::TimelineEvent &event,
+  const mtx::events::collections::TimelineEvents &event,
   const RoomContext &ctx,
-  const std::vector<std::pair<mtx::common::Relation, mtx::events::collections::TimelineEvent>>
+  const std::vector<std::pair<mtx::common::Relation, mtx::events::collections::TimelineEvents>>
     &relatedEvents) const
 {
     auto event_json = nlohmann::json(event);
