@@ -102,19 +102,23 @@ parse_room_account_data_events(
     container.reserve(events.size());
 
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
-        // if (!e.contains("content") || e["content"].empty()) {
-        //        container.emplace_back(events::AccountDataEvent<events::msg::Redacted>(e));
-        //        break;
-        //}
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
+            // if (!e.contains("content") || e["content"].empty()) {
+            //        container.emplace_back(events::AccountDataEvent<events::msg::Redacted>(e));
+            //        break;
+            //}
+
             switch (type) {
             case events::EventType::Direct: {
                 container.emplace_back(events::AccountDataEvent<Direct>(e));
                 break;
             }
+            case events::EventType::IgnoredUsers: {
+                container.emplace_back(events::AccountDataEvent<IgnoredUsers>(e));
+                break;
+            }
             case events::EventType::Tag: {
                 container.emplace_back(events::AccountDataEvent<Tags>(e));
                 break;
@@ -132,6 +136,10 @@ parse_room_account_data_events(
                 container.emplace_back(events::AccountDataEvent<nheko_extensions::HiddenEvents>(e));
                 break;
             }
+            case events::EventType::NhekoEventExpiry: {
+                container.emplace_back(events::AccountDataEvent<nheko_extensions::EventExpiry>(e));
+                break;
+            }
             case events::EventType::ImagePackRooms: {
                 container.emplace_back(
                   events::AccountDataEvent<events::msc2545::ImagePackRooms>(e));
@@ -177,6 +185,7 @@ parse_room_account_data_events(
             case events::EventType::RoomPowerLevels:
             case events::EventType::RoomRedaction:
             case events::EventType::RoomTombstone:
+            case events::EventType::RoomServerAcl:
             case events::EventType::RoomTopic:
             case events::EventType::Widget:
             case events::EventType::VectorWidget:
@@ -220,9 +229,9 @@ parse_timeline_events(const json &events,
     container.reserve(events.size());
 
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
             if (e.contains("unsigned") && e["unsigned"].contains("redacted_by")) {
                 if (e.contains("state_key"))
                     container.emplace_back(events::StateEvent<events::msg::Redacted>(e));
@@ -299,6 +308,11 @@ parse_timeline_events(const json &events,
 
                 break;
             }
+            case events::EventType::RoomServerAcl: {
+                container.emplace_back(events::StateEvent<ServerAcl>(e));
+
+                break;
+            }
             case events::EventType::RoomTopic: {
                 container.emplace_back(events::StateEvent<Topic>(e));
 
@@ -389,8 +403,8 @@ parse_timeline_events(const json &events,
 
                     break;
                 }
-                case MsgType::Confetti: {
-                    container.emplace_back(events::RoomEvent<events::msg::Confetti>(e));
+                case MsgType::ElementEffect: {
+                    container.emplace_back(events::RoomEvent<events::msg::ElementEffect>(e));
 
                     break;
                 }
@@ -436,9 +450,15 @@ parse_timeline_events(const json &events,
                     break;
                 }
                 case MsgType::Unknown: {
+                    container.emplace_back(events::RoomEvent<events::msg::Unknown>(e));
+                    break;
+                }
+                case MsgType::Redacted: {
                     container.emplace_back(events::RoomEvent<events::Unknown>(e));
                     break;
                 }
+                case MsgType::Invalid:
+                    break;
                 }
                 break;
             }
@@ -505,7 +525,9 @@ parse_timeline_events(const json &events,
             case events::EventType::Typing:
             case events::EventType::Receipt:
             case events::EventType::FullyRead:
+            case events::EventType::IgnoredUsers:
             case events::EventType::NhekoHiddenEvents:
+            case events::EventType::NhekoEventExpiry:
             case events::EventType::ImagePackRooms:
             case events::EventType::ImagePackInAccountData:
             case events::EventType::Dummy:
@@ -524,9 +546,9 @@ parse_device_events(const json &events,
     container.clear();
     container.reserve(events.size());
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
             switch (type) {
             case events::EventType::RoomEncrypted: {
                 const auto algo = e.at("content").at("algorithm").get<std::string>();
@@ -623,9 +645,9 @@ parse_state_events(const json &events,
     container.reserve(events.size());
 
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
             if (e.contains("unsigned") && e["unsigned"].contains("redacted_by")) {
                 container.emplace_back(events::StateEvent<events::msg::Redacted>(e));
                 continue;
@@ -691,6 +713,11 @@ parse_state_events(const json &events,
 
                 break;
             }
+            case events::EventType::RoomServerAcl: {
+                container.emplace_back(events::StateEvent<ServerAcl>(e));
+
+                break;
+            }
             case events::EventType::RoomTopic: {
                 container.emplace_back(events::StateEvent<Topic>(e));
 
@@ -778,7 +805,9 @@ parse_state_events(const json &events,
             case events::EventType::Typing:
             case events::EventType::Receipt:
             case events::EventType::FullyRead:
+            case events::EventType::IgnoredUsers:
             case events::EventType::NhekoHiddenEvents:
+            case events::EventType::NhekoEventExpiry:
             case events::EventType::ImagePackRooms:
             case events::EventType::ImagePackInAccountData:
             case events::EventType::Dummy:
@@ -798,9 +827,9 @@ parse_stripped_events(const json &events,
     container.reserve(events.size());
 
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
             switch (type) {
             case events::EventType::RoomAliases: {
                 container.emplace_back(events::StrippedEvent<mtx::events::state::Aliases>(e));
@@ -857,6 +886,11 @@ parse_stripped_events(const json &events,
 
                 break;
             }
+            case events::EventType::RoomServerAcl: {
+                container.emplace_back(events::StrippedEvent<ServerAcl>(e));
+
+                break;
+            }
             case events::EventType::RoomTopic: {
                 container.emplace_back(events::StrippedEvent<Topic>(e));
 
@@ -940,7 +974,9 @@ parse_stripped_events(const json &events,
             case events::EventType::Typing:
             case events::EventType::Receipt:
             case events::EventType::FullyRead:
+            case events::EventType::IgnoredUsers:
             case events::EventType::NhekoHiddenEvents:
+            case events::EventType::NhekoEventExpiry:
             case events::EventType::ImagePackInAccountData:
             case events::EventType::ImagePackInRoom:
             case events::EventType::ImagePackRooms:
@@ -960,9 +996,9 @@ parse_ephemeral_events(const json &events,
     container.clear();
     container.reserve(events.size());
     for (const auto &e : events) {
-        const auto type = mtx::events::getEventType(e);
-
         try {
+            const auto type = mtx::events::getEventType(e);
+
             switch (type) {
             case events::EventType::Typing: {
                 container.emplace_back(events::EphemeralEvent<events::ephemeral::Typing>(e));
