@@ -1,3 +1,4 @@
+#include <algorithm>
 #include <gtest/gtest.h>
 
 #include <mtx.hpp>
@@ -54,10 +55,10 @@ TEST(Events, Redacted)
           "type": "m.room.redaction"
         })"_json;
 
-    mtx::events::collections::TimelineEvent event =
-      data.get<mtx::events::collections::TimelineEvent>();
+    mtx::events::collections::TimelineEvents event =
+      data.get<mtx::events::collections::TimelineEvents>();
 
-    ASSERT_TRUE(std::holds_alternative<ns::RedactionEvent<ns::msg::Redaction>>(event.data));
+    ASSERT_TRUE(std::holds_alternative<ns::RedactionEvent<ns::msg::Redaction>>(event));
 
     json data2 = R"({
 		"content": {
@@ -84,13 +85,13 @@ TEST(Events, Redacted)
 		"event_id": "$redacted_id_2"
 	})"_json;
 
-    event = data2.get<mtx::events::collections::TimelineEvent>();
-    ASSERT_TRUE(std::holds_alternative<ns::StateEvent<ns::msg::Redacted>>(event.data));
-    ASSERT_TRUE(std::get<ns::StateEvent<ns::msg::Redacted>>(event.data)
+    event = data2.get<mtx::events::collections::TimelineEvents>();
+    ASSERT_TRUE(std::holds_alternative<ns::StateEvent<ns::msg::Redacted>>(event));
+    ASSERT_TRUE(std::get<ns::StateEvent<ns::msg::Redacted>>(event)
                   .unsigned_data.redacted_because.has_value());
-    ASSERT_EQ(std::get<ns::StateEvent<ns::msg::Redacted>>(event.data)
-                .unsigned_data.redacted_because->sender,
-              "@redacted_user_2:example.com");
+    ASSERT_EQ(
+      std::get<ns::StateEvent<ns::msg::Redacted>>(event).unsigned_data.redacted_because->sender,
+      "@redacted_user_2:example.com");
 }
 
 TEST(Events, Conversions)
@@ -246,7 +247,7 @@ TEST(StateEvents, Create)
     EXPECT_EQ(event.unsigned_data.age, 3715756343L);
     EXPECT_EQ(event.origin_server_ts, 1506761923948L);
     EXPECT_EQ(event.state_key, "");
-    EXPECT_EQ(event.content.creator, "@mujx:matrix.org");
+    // EXPECT_EQ(event.content.creator, "@mujx:matrix.org");
 
     json example_from_spec = R"({
             "content": {
@@ -277,7 +278,7 @@ TEST(StateEvents, Create)
     EXPECT_EQ(event.unsigned_data.age, 1234);
     EXPECT_EQ(event.origin_server_ts, 1432735824653L);
     EXPECT_EQ(event.state_key, "");
-    EXPECT_EQ(event.content.creator, "@example:example.org");
+    // EXPECT_EQ(event.content.creator, "@example:example.org");
     EXPECT_EQ(event.content.federate, true);
     EXPECT_EQ(event.content.room_version, "1");
     EXPECT_EQ(event.content.predecessor->room_id, "!oldroom:example.org");
@@ -309,7 +310,7 @@ TEST(StateEvents, CreateWithType)
     EXPECT_EQ(event.unsigned_data.age, 3715756343L);
     EXPECT_EQ(event.origin_server_ts, 1506761923948L);
     EXPECT_EQ(event.state_key, "");
-    EXPECT_EQ(event.content.creator, "@mujx:matrix.org");
+    // EXPECT_EQ(event.content.creator, "@mujx:matrix.org");
     EXPECT_TRUE(event.content.type.has_value());
     EXPECT_EQ(event.content.type.value(), ns::state::room_type::space);
 
@@ -807,6 +808,54 @@ TEST(StateEvents, Tombstone)
     EXPECT_EQ(event.content.replacement_room, "!newroom:example.org");
 }
 
+TEST(StateEvents, ServerAcl)
+{
+    json data = R"(
+{
+  "content": {
+    "allow": [
+      "*"
+    ],
+    "allow_ip_literals": false,
+    "deny": [
+      "*.evil.com",
+      "evil.com"
+    ]
+  },
+  "event_id": "$143273582443PhrSn:example.org",
+  "origin_server_ts": 1432735824653,
+  "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
+  "sender": "@example:example.org",
+  "state_key": "",
+  "type": "m.room.server_acl",
+  "unsigned": {
+    "age": 1234
+  }
+}
+)"_json;
+
+    ns::StateEvent<ns::state::ServerAcl> event = data.get<ns::StateEvent<ns::state::ServerAcl>>();
+
+    EXPECT_EQ(event.type, ns::EventType::RoomServerAcl);
+    EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
+    EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
+    EXPECT_EQ(event.sender, "@example:example.org");
+    EXPECT_EQ(event.origin_server_ts, 1432735824653);
+    EXPECT_EQ(event.unsigned_data.age, 1234);
+    EXPECT_EQ(event.state_key, "");
+    EXPECT_EQ(event.content.allow,
+              (std::vector<std::string>{
+                "*",
+              }));
+    EXPECT_EQ(event.content.deny,
+              (std::vector<std::string>{
+                "*.evil.com",
+                "evil.com",
+              }));
+    EXPECT_EQ(event.content.allow_ip_literals, false);
+    EXPECT_EQ(data.dump(), nlohmann::json(event).dump());
+}
+
 TEST(StateEvents, Topic)
 {
     json data = R"({
@@ -1525,6 +1574,27 @@ TEST(Ephemeral, Receipt)
     EXPECT_EQ(j.dump(), json(event).dump());
 }
 
+TEST(AccountData, Faulty)
+{
+    json j = R"(
+    [
+      {
+        "content": {
+          "@bob:example.com": [
+             "!abcdefgh:example.com",
+             "!hgfedcba:example.com"
+           ]
+         },
+         "type": ""
+      }
+     ])"_json;
+
+    mtx::responses::AccountData events;
+    EXPECT_NO_THROW(mtx::responses::utils::parse_room_account_data_events(j, events.events));
+
+    EXPECT_EQ(events.events.size(), 0);
+}
+
 TEST(AccountData, Direct)
 {
     json j = R"({
@@ -1567,6 +1637,38 @@ TEST(AccountData, FullyRead)
     EXPECT_EQ(j.dump(), json(event).dump());
 }
 
+TEST(AccountData, IgnoredUsers)
+{
+    json j = R"({
+    "content": {
+        "ignored_users": {
+            "@someone:example.org": {},
+            "@anotherone:example.org": {}
+        }
+    },
+    "type": "m.ignored_user_list"
+})"_json;
+    ns::AccountDataEvent<ns::account_data::IgnoredUsers> event =
+      j.get<ns::AccountDataEvent<ns::account_data::IgnoredUsers>>();
+
+    ASSERT_EQ(event.content.users.size(), 2);
+
+    // JSON does not demand a guaranteed key order
+    ASSERT_TRUE(std::any_of(event.content.users.cbegin(),
+                            event.content.users.cend(),
+                            [](const ns::account_data::IgnoredUser &entry) {
+                                return entry.id == "@someone:example.org";
+                            }));
+    ASSERT_TRUE(std::any_of(event.content.users.cbegin(),
+                            event.content.users.cend(),
+                            [](const ns::account_data::IgnoredUser &entry) {
+                                return entry.id == "@anotherone:example.org";
+                            }));
+
+    ASSERT_EQ(event.type, ns::EventType::IgnoredUsers);
+    ASSERT_EQ(j.dump(), json(event).dump());
+}
+
 TEST(ToDevice, KeyVerificationRequest)
 {
     json request_data = R"({
@@ -1578,7 +1680,7 @@ TEST(ToDevice, KeyVerificationRequest)
         "timestamp": 1559598944869,
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.request"
 })"_json;
 
@@ -1615,7 +1717,7 @@ TEST(ToDevice, KeyVerificationStart)
         ],
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.start"
 })"_json;
 
@@ -1651,7 +1753,7 @@ TEST(ToDevice, KeyVerificationAccept)
         ],
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.accept"
 })"_json;
 
@@ -1719,7 +1821,7 @@ TEST(ToDevice, KeyVerificationCancel)
         "reason": "User rejected the key verification request",
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.cancel"
 })"_json;
 
@@ -1740,7 +1842,7 @@ TEST(ToDevice, KeyVerificationKey)
         "key": "fQpGIW1Snz+pwLZu6sTy2aHy/DYWWTspTJRPyNp0PKkymfIsNffysMl6ObMMFdIJhk6g6pwlIqZ54rxo8SLmAg",
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.key"
 })"_json;
 
@@ -1765,7 +1867,7 @@ TEST(ToDevice, KeyVerificationMac)
         },
         "transaction_id": "S0meUniqueAndOpaqueString"
     },
-    "sender": "",
+    "sender": "@mujx:matrix.org",
     "type": "m.key.verification.mac"
 })"_json;
 
@@ -1857,10 +1959,10 @@ TEST(Collection, Events)
 	  "type": "m.room.aliases"
 	})"_json;
 
-    mtx::events::collections::TimelineEvent event =
-      data.get<mtx::events::collections::TimelineEvent>();
+    mtx::events::collections::TimelineEvents event =
+      data.get<mtx::events::collections::TimelineEvents>();
 
-    ASSERT_TRUE(std::get_if<ns::StateEvent<ns::state::Aliases>>(&event.data) != nullptr);
+    ASSERT_TRUE(std::get_if<ns::StateEvent<ns::state::Aliases>>(&event) != nullptr);
 }
 
 TEST(RoomAccountData, Tags)
@@ -1898,8 +2000,8 @@ TEST(RoomAccountData, NhekoHiddenEvents)
     json data = R"({
           "content": {
               "hidden_event_types": [
-	          "m.reaction",
-		  "m.room.member"
+              "m.reaction",
+              "m.room.member"
 	      ]
           },
           "type": "im.nheko.hidden_events"
@@ -1913,6 +2015,56 @@ TEST(RoomAccountData, NhekoHiddenEvents)
     ASSERT_EQ(event.content.hidden_event_types->size(), 2);
     EXPECT_EQ(event.content.hidden_event_types.value()[0], ns::EventType::Reaction);
     EXPECT_EQ(event.content.hidden_event_types.value()[1], ns::EventType::RoomMember);
+
+    json data2 = R"({
+          "content": {
+              "hidden_event_types": [
+              ""
+	      ]
+          },
+          "type": "im.nheko.hidden_events"
+        })"_json;
+    ns::AccountDataEvent<ns::account_data::nheko_extensions::HiddenEvents> event2 =
+      data2.get<ns::AccountDataEvent<ns::account_data::nheko_extensions::HiddenEvents>>();
+    ASSERT_TRUE(event2.content.hidden_event_types.has_value());
+    ASSERT_EQ(event2.content.hidden_event_types.value().size(), 1);
+    EXPECT_EQ(event2.content.hidden_event_types.value().front(),
+              mtx::events::EventType::Unsupported);
+}
+
+TEST(RoomAccountData, NhekoEventExpiry)
+{
+    json data = R"({
+          "content": {
+          },
+          "type": "im.nheko.event_expiry"
+        })"_json;
+
+    ns::AccountDataEvent<ns::account_data::nheko_extensions::EventExpiry> event =
+      data.get<ns::AccountDataEvent<ns::account_data::nheko_extensions::EventExpiry>>();
+
+    EXPECT_EQ(event.type, ns::EventType::NhekoEventExpiry);
+    ASSERT_FALSE(event.content.exclude_state_events);
+    ASSERT_EQ(event.content.expire_after_ms, 0);
+    ASSERT_EQ(event.content.protect_latest, 0);
+    ASSERT_EQ(event.content.keep_only_latest, 0);
+
+    data = R"({
+          "content": {
+          "exclude_state_events": true,
+          "expire_after_ms": 100,
+          "protect_latest": 100,
+          "keep_only_latest": 100
+          },
+          "type": "im.nheko.event_expiry"
+        })"_json;
+
+    event = data.get<ns::AccountDataEvent<ns::account_data::nheko_extensions::EventExpiry>>();
+
+    ASSERT_TRUE(event.content.exclude_state_events);
+    ASSERT_EQ(event.content.expire_after_ms, 100);
+    ASSERT_EQ(event.content.protect_latest, 100);
+    ASSERT_EQ(event.content.keep_only_latest, 100);
 }
 
 TEST(RoomAccountData, ImagePack)
