@@ -119,8 +119,9 @@ TEST(RoomEvents, AudioMessage)
               mtx::common::RelationType::InReplyTo);
 }
 
-TEST(RoomEvents, ConfettiMessage)
+TEST(RoomEvents, ElementEffectMessage)
 {
+    // confetti
     json data = R"({
           "origin_server_ts": 1510489356530,
           "sender": "@nheko_test:matrix.org",
@@ -142,7 +143,7 @@ TEST(RoomEvents, ConfettiMessage)
           "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
          })"_json;
 
-    RoomEvent<msg::Confetti> event = data.get<RoomEvent<msg::Confetti>>();
+    RoomEvent<msg::ElementEffect> event = data.get<RoomEvent<msg::ElementEffect>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15104893562785758wEgEU:matrix.org");
@@ -160,6 +161,35 @@ TEST(RoomEvents, ConfettiMessage)
               mtx::common::RelationType::InReplyTo);
 
     EXPECT_EQ(data.dump(), json(event).dump());
+
+    // space invaders
+    data = R"({
+          "origin_server_ts": 1510489356530,
+          "sender": "@nheko_test:matrix.org",
+          "event_id": "$15104893562785758wEgEU:matrix.org",
+          "unsigned": {
+            "age": 2225,
+            "transaction_id": "m1510489356267.2"
+          },
+          "content": {
+            "body": "galaga > space invaders",
+            "msgtype": "io.element.effects.space_invaders",
+          "m.relates_to": {
+          "m.in_reply_to": {
+                       "event_id": "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E"
+                  }
+              }
+          },
+          "type": "m.room.message",
+          "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
+         })"_json;
+
+    event = data.get<RoomEvent<msg::ElementEffect>>();
+
+    EXPECT_EQ(event.type, EventType::RoomMessage);
+    EXPECT_EQ(event.content.body, "galaga > space invaders");
+    EXPECT_EQ(event.content.msgtype, "io.element.effects.space_invaders");
+    EXPECT_EQ(data.dump(), json(event).dump());
 }
 
 TEST(RoomEvents, EmoteMessage)
@@ -557,6 +587,57 @@ TEST(RoomEvents, TextMessage)
     EXPECT_EQ(data.dump(), json(event).dump());
 }
 
+TEST(RoomEvents, TextMessageWithMention)
+{
+    json data = R"({
+          "origin_server_ts": 1510489356530,
+          "sender": "@nheko_test:matrix.org",
+          "event_id": "$15104893562785758wEgEU:matrix.org",
+          "unsigned": {
+            "age": 2225,
+            "transaction_id": "m1510489356267.2"
+          },
+          "content": {
+            "body": "hey there",
+            "msgtype": "m.text",
+	      "m.relates_to": {
+		  "m.in_reply_to": {
+                       "event_id": "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E"
+                  }
+              },
+              "m.mentions": {
+                "room": true,
+                "user_ids": ["@alice:example.org"]
+              }
+          },
+          "type": "m.room.message",
+          "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
+         })"_json;
+
+    RoomEvent<msg::Text> event = data.get<RoomEvent<msg::Text>>();
+
+    EXPECT_EQ(event.type, EventType::RoomMessage);
+    EXPECT_EQ(event.event_id, "$15104893562785758wEgEU:matrix.org");
+    EXPECT_EQ(event.room_id, "!lfoDRlNFWlvOnvkBwQ:matrix.org");
+    EXPECT_EQ(event.sender, "@nheko_test:matrix.org");
+    EXPECT_EQ(event.origin_server_ts, 1510489356530L);
+    EXPECT_EQ(event.unsigned_data.age, 2225);
+    EXPECT_EQ(event.unsigned_data.transaction_id, "m1510489356267.2");
+
+    EXPECT_EQ(event.content.body, "hey there");
+    EXPECT_EQ(event.content.msgtype, "m.text");
+    EXPECT_EQ(event.content.relations.relations.at(0).event_id,
+              "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E");
+    EXPECT_EQ(event.content.relations.relations.at(0).rel_type,
+              mtx::common::RelationType::InReplyTo);
+
+    ASSERT_TRUE(event.content.mentions.has_value());
+    EXPECT_EQ(event.content.mentions->room, true);
+    EXPECT_EQ(event.content.mentions->user_ids, std::vector<std::string>{"@alice:example.org"});
+
+    EXPECT_EQ(data.dump(), json(event).dump());
+}
+
 TEST(RoomEvents, VideoMessage)
 {
     json data = R"({
@@ -856,3 +937,97 @@ TEST(RoomEvents, ThreadedMessage)
 
     EXPECT_EQ(data.dump(), json(event).dump());
 }
+
+TEST(RoomEvents, UnknownMessage)
+{
+    json data = R"({
+          "origin_server_ts": 1510489356530,
+          "sender": "@nheko_test:matrix.org",
+          "event_id": "$15104893562785758wEgEU:matrix.org",
+          "unsigned": {
+            "age": 2225,
+            "transaction_id": "m1510489356267.2"
+          },
+          "content": {
+            "body": "party!",
+            "msgtype": "like.its.1999",
+          "m.relates_to": {
+          "m.in_reply_to": {
+                       "event_id": "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E"
+                  }
+              }
+          },
+          "type": "m.room.message",
+          "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
+         })"_json;
+
+    RoomEvent<msg::Unknown> event = data.get<RoomEvent<msg::Unknown>>();
+
+    EXPECT_EQ(event.type, EventType::RoomMessage);
+    EXPECT_EQ(event.event_id, "$15104893562785758wEgEU:matrix.org");
+    EXPECT_EQ(event.room_id, "!lfoDRlNFWlvOnvkBwQ:matrix.org");
+    EXPECT_EQ(event.sender, "@nheko_test:matrix.org");
+    EXPECT_EQ(event.origin_server_ts, 1510489356530L);
+    EXPECT_EQ(event.unsigned_data.age, 2225);
+    EXPECT_EQ(event.unsigned_data.transaction_id, "m1510489356267.2");
+
+    EXPECT_EQ(event.content.body, "party!");
+    EXPECT_EQ(event.content.msgtype, "like.its.1999");
+    EXPECT_EQ(event.content.relations.relations.at(0).event_id,
+              "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E");
+    EXPECT_EQ(event.content.relations.relations.at(0).rel_type,
+              mtx::common::RelationType::InReplyTo);
+
+    EXPECT_EQ(data.dump(), json(event).dump());
+}
+
+TEST(RoomEvents, RedactedMessage)
+{
+    json data = R"({
+          "origin_server_ts": 1510489356530,
+          "sender": "@nheko_test:matrix.org",
+          "event_id": "$15104893562785758wEgEU:matrix.org",
+          "unsigned": {
+            "age": 2225,
+            "transaction_id": "m1510489356267.2"
+          },
+          "content": null,
+          "type": "m.room.message",
+          "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
+    })"_json;
+
+    auto messageType         = getMessageType(data["content"]);
+    RoomEvent<Unknown> event = data.get<RoomEvent<Unknown>>();
+
+    EXPECT_EQ(event.type, EventType::RoomMessage);
+    EXPECT_EQ(messageType, MessageType::Redacted);
+}
+
+TEST(RoomEvents, InvalidMessage)
+{
+    json data = R"({
+          "origin_server_ts": 1510489356530,
+          "sender": "@nheko_test:matrix.org",
+          "event_id": "$15104893562785758wEgEU:matrix.org",
+          "unsigned": {
+            "age": 2225,
+            "transaction_id": "m1510489356267.2"
+          },
+          "content": {
+            "body": "I don't have a msgtype :(",
+            "m.relates_to": {
+              "m.in_reply_to": {
+                "event_id": "$6GKhAfJOcwNd69lgSizdcTob8z2pWQgBOZPrnsWMA1E"
+              }
+            }
+          },
+          "type": "m.room.message",
+          "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
+    })"_json;
+
+    auto messageType         = getMessageType(data["content"]);
+    RoomEvent<Unknown> event = data.get<RoomEvent<Unknown>>();
+
+    EXPECT_EQ(event.type, EventType::RoomMessage);
+    EXPECT_EQ(messageType, MessageType::Invalid);
+}
